﻿Type=StaticCode
Version=6.8
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@


Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

End Sub

#Region "FreqAnalyser 1"

'Find a good position to compare wave with the first periode
Sub GetBestComparer(periodes() As Int) As Int
	Dim iWL1, iWL2 As Int
	iWL1 = periodes(1) - periodes(0)
	If iWL1 < 10 Then Return 0
	Dim iLast As Int = periodes.Length - 2 'never the very last, because we still need a wavelenght to compare it
	For i = 2 To iLast
		iWL2 = periodes(i) - periodes(i-1)
		If Abs(iWL1 - iWL2) >= 10 Then Return i - 2 'this is the first bad one, so take the last
	Next
	Return iLast
End Sub

'finds the Position (somewhere near of a peak) where the wave is the best copy of Pos1
Sub GetCorrector(wave() As Short, pos1 As Int, pos2 As Int, lenght As Int) As Int
	Dim iDiff, iCorrectorDiff, iCorrector As Int
	iDiff =  WaveCompare(wave, pos1, pos2, lenght)
	For j = 1 To 5
		iCorrectorDiff = WaveCompare(wave, pos1, pos2 + j, lenght)
		If iCorrectorDiff >= iDiff Then
			iCorrector = j - 1
			Exit
		Else
			iCorrector = j
			iDiff = iCorrectorDiff
		End If
	Next
	If iCorrector = 0 Then
		For j = -1 To -5 Step -1
			iCorrectorDiff = WaveCompare(wave, pos1, pos2 + j, lenght)
			If iCorrectorDiff >= iDiff Then
				iCorrector = j + 1
				Exit
			Else
				iCorrector = j
				iDiff = iCorrectorDiff
			End If
		Next
	End If
	Return iCorrector
End Sub

Sub WaveCompare(wave() As Short, pos1 As Int, pos2 As Int, lenght As Int) As Int
	Dim iDifference As Int
	Dim iDistance As Int = pos2 - pos1
	If pos2 + iDistance + lenght >= wave.Length Then Return 999999
	For i = pos1 To pos1 + lenght
		iDifference = iDifference + Abs(wave(i) - wave(i+iDistance))
	Next
	Return iDifference
End Sub

Sub GetWavelenghtFromPeriodes(periodes() As Int) As Int
	Dim iAnz, iPeak, iWL, iWLSum As Int
	For i = 2 To periodes.Length - 1
		iPeak = periodes(i)
		If iPeak = 0 Then Exit
		iWL = Abs(iPeak - periodes(i-1))
		iWLSum = iWLSum + iWL
		iAnz = iAnz + 1
	Next
	Return Round(iWLSum / iAnz)
End Sub

Sub TestGetPeriodes(iPeriodes() As Int) As String
	Dim sb As StringBuilder
	sb.Initialize
	If iPeriodes(0) = 0 Then Return ""
	sb.Append("(0) = (" & iPeriodes(0) & ")" & CRLF & CRLF)
	For i = 1 To iPeriodes.Length - 1
		If iPeriodes(i) > 0 Then
			sb.Append("(" & i & ") = " & (iPeriodes(i) - iPeriodes(i-1)) & " (" & iPeriodes(i) & ")" & CRLF)
		Else
			sb.Append("(" & i & ") = 0" & CRLF)
		End If
	Next
	'sb.Append(CRLF &  "Wavelenght: " & GetWavelenghtFromPeriodes(iPeriodes))
	Return sb.ToString
End Sub

'returns the start positions of periodes on a sound wave array (only the fundamental frequence, not the overtones)
 Sub GetPeriodes(wave() As Short) As Int()
	Dim iPartToCheck As Int = 2000, iMaxPeriodes As Short = 20' 12
	Dim iPeriodes(iMaxPeriodes), iPeriodeCounter, iWL As Int
	Dim i, iX, iEnd As Int
	Dim iY, iMin, iMax, iAmplitude, iTempAmpl, iMinAmpl, iMaxAmpl As Short
	Dim iStatus As Short 'Wave-Status: -1 comes from lower area, 1 comes from upper area
	Dim iShortcut As Int = 1 'After first periode just jump to the next! good values: 1=ON, 1000=OFF '1
	iEnd = Min(iPartToCheck, wave.Length - 1)
	
	For i = 0 To iEnd
		iY = wave(i)
		
		If iY >= 0 Then 'upper area
			iTempAmpl = iY - iMin 'Ampitude is the best way to distinguish fundamental and overtone periodes (min, max can change)
			If iStatus = -1 Then ' first positive Y after negative peak
				iX = i '! take as a unique periode start, but save only at positive peak, to avoid overtone periodes
				iStatus = 0
				iMinAmpl = (iMax - iMin) * 0.9 'update the switch! The closer to 1 the safer to avoid overtone period, but risk of losing basic period
				iMax = 0 'reset max before the new peak
				If iPeriodeCounter >= iShortcut Then 'after first periodes we can be faster
					iWL =  iX - iPeriodes(iPeriodeCounter - 1)
					If iWL > 5 Then
						iPeriodes(iPeriodeCounter) = iX
						iPeriodeCounter = iPeriodeCounter + 1
						iStatus = - 1
						i = i + iWL - 5
					End If
				End If
			else If iTempAmpl >= iMinAmpl Then 'positive peak
				iStatus = 1
				If iY > iMax Then iMax = iY
				If iTempAmpl > iAmplitude Then iAmplitude = iTempAmpl 'only for getting the volume
				If iX > 0 Then 'we allready have an X position of next periode start
					If iTempAmpl > iMaxAmpl Then 'Reset (if we started with too small aplitudes because of overtone periodes)
						iPeriodeCounter = 0
						iMinAmpl = iTempAmpl * 0.9 'The closer to 1 the safer to avoid overtone period, but risk of losing basic period
						iMaxAmpl = iTempAmpl * 1.1
					Else
						iPeriodes(iPeriodeCounter) = iX 'save the new X (now we know, that it was not only a overtone periode)
						iPeriodeCounter = iPeriodeCounter + 1
						If iPeriodeCounter >= iMaxPeriodes Then Exit 'we have enought periods (quit higher frequences that way)
						iMin = 0
						iX = 0 'again: prepare at negative peak, take at zero, save at positive peak
					End If
				End If
			End If
		Else 'lower area
			iTempAmpl = iMax - iY
			If iY < iMin Then iMin = iY
			If iTempAmpl >= iMinAmpl  Then
				iStatus = - 1 'now we wait for a unique position of every period, close to zero line
			End If
		End If
		
		If iPeriodeCounter >= iMaxPeriodes Then Exit 'we have enought periods (quit higher frequences that way)
	Next
	If iPeriodeCounter < iMaxPeriodes Then iPeriodes(iPeriodeCounter) = 0 'mark the end
	If iAmplitude < 1000 Then iPeriodes(0) = 0 'too quiet
	
	Return iPeriodes
End Sub

#End Region